import json
from typing import Any, Dict, List

import requests
from loguru import logger
from PIL import Image
from pymemri.data.schema import CategoricalPrediction, Photo
from pymemri.plugin.pluginbase import PluginBase
from pymemri.webserver.public_api import register_endpoint

from .model import DEFAULT_MODEL, ImageClassifier

DEFAULT_MODEL

# NOTE If you change the plugin class name, be sure to edit the `pluginName` in `/metadata.json`
class ImageTaggingPlugin(PluginBase):
    def __init__(self, model_name: str = DEFAULT_MODEL, **kwargs):
        super().__init__(**kwargs)
        self.model = ImageClassifier(model_name=model_name)
    
    def run(self):
        logger.info("Starting plugin")
        self.daemon = True

    def add_to_schema(self):
        raise NotImplementedError()

    @register_endpoint("/v1/classify_ids", method="POST")
    def classify_ids(self, item_ids: List[str]) -> List[Dict[str, Any]]:        
        photos = [self.client.get_photo(uid) for uid in item_ids]
        images = [photo.to_PIL() for photo in photos]
        predictions = self.model.predict(images)
        self._sync_to_pod(photos, predictions)
        return predictions

    @register_endpoint("/v1/classify_urls", method="POST")
    def classify_urls(self, urls: List[str]) -> List[Dict[str, Any]]:
        images = [self._image_from_url(url) for url in urls]
        result = self.model.predict(images)
        return result

    @staticmethod
    def _prediction_to_item(prediction: Dict[str, Any]) -> CategoricalPrediction:
        return CategoricalPrediction(
            value=json.dumps([prediction["label"]]),
            probs=json.dumps([prediction["score"]])
        )

    def _sync_to_pod(self, photos: List[Photo], predictions: List[Dict[str, Any]]) -> None:
        edges = []
        items = []
        for photo, prediction in zip(photos, predictions):
            pred_item = self._prediction_to_item(prediction)
            items.append(pred_item)
            edges.append(photo.add_edge("label", pred_item))
        self.client.bulk_action(create_items=items, create_edges=edges)

    @staticmethod
    def _image_from_url(url: str) -> Image.Image:
        return Image.open(requests.get(url, stream=True).raw)

    def add_to_schema(self):
        self.client.add_to_schema(Photo, CategoricalPrediction)