import json
from typing import List

import pytest
import requests
from PIL import Image
from pymemri.data.schema import Photo
from pymemri.pod.client import PodClient

from image_tagging.plugin import ImageTaggingPlugin

# First five images from COCO val 2017
IMAGE_URLS = [
    'http://images.cocodataset.org/val2017/000000397133.jpg',
    'http://images.cocodataset.org/val2017/000000037777.jpg',
    'http://images.cocodataset.org/val2017/000000252219.jpg',
    'http://images.cocodataset.org/val2017/000000087038.jpg',
    'http://images.cocodataset.org/val2017/000000174482.jpg',
]

@pytest.fixture
def images() -> List[Image.Image]:
    return [Image.open(requests.get(url, stream=True).raw) for url in IMAGE_URLS]

@pytest.fixture(scope="session")
def client():
    client = PodClient()
    client.add_to_schema(Photo)
    return client

@pytest.fixture
def config():
    with open("./config.json") as f:
        content = json.load(f)
    return content

@pytest.fixture
def metadata():
    with open("./metadata.json") as f:
        content = json.load(f)
    return content

def create_photo_items(client: PodClient, images: List[Image.Image]) -> List[str]:

    photos = [Photo.from_PIL(image) for image in images]
    for photo in photos:
        client.create_photo(photo, asyncFlag=False)
    return photos

def test_classify_ids(client: PodClient, images: List[Image.Image]):
    photos = create_photo_items(client, images)
    ids = [photo.id for photo in photos]

    plugin = ImageTaggingPlugin(client=client)
    plugin.add_to_schema()
    predictions = plugin.classify_ids(ids)
    
    assert len(predictions) == len(ids)

    client.reset_local_db()
    photo_items = client.search({"ids": ids})
    assert len(photo_items) == len(ids)
    for p in photo_items:
        assert len(p.label) == 1

def test_classify_urls(client: PodClient):
    plugin = ImageTaggingPlugin(client=client)
    plugin.add_to_schema()
    predictions = plugin.classify_urls(IMAGE_URLS)

    assert len(predictions) == len(IMAGE_URLS)
    for p in predictions:
        assert isinstance(p, dict) and len(p) == 2
        assert "label" in p and "score" in p
